#####################################################################################################################
# Copyright IBM
#
# This PERL script was created by Stuart Poulin Dango of IBM Rational Software.
# 
#
# Updated by: Stuart Poulin
# Updated on: 20080917
######################################################################################################################
# 
#This script utilized the ClearQuest COM API and the ClearCase cleartool commands.
# 
# Variable used in this script are as followings:
#	user		The login id of the ClearQuest user
#	pw			The password of the ClearQuest user
#	dbname		The schema repository connection to the database
#	dbset		The user database connection
#	baseline	The name of the baseline record
#	pvob		The name of the project vob for this baseline
#	logfile			The path where the log file will be sent
#	test			This is a flag that can be set for testing file inputs
#	help			This is a flag for the help
#######################################################################################################################
#
# Change Log:
# Flag      Item                Date        Programmer         Comments
#
#######################################################################################################################
#!ratlperl -l -w
# Changed above line to read ratlperl from cqperl thl1
use strict; 
use vars qw($Bin);
use FindBin qw($Bin);
use lib "$Bin";
use Getopt::Long;
use File::Basename;
use Carp;
use FileHandle;
use File::Spec;  
use CQPerlExt;
use Env;
use LWP::Simple;
use POSIX;

use get_record_details;
use create_record;
my $promotion_level;
my $slashvob;
my $noslashvob;
my ($entity);
my ($failure);
my ($status,$resultSetStatus); #thl2
use subs qw(get_promotionlevel main);
# Unbuffer STDERR and STDOUT
select STDERR;  
$| = 1;
select STDOUT;
$| = 1;

#Define Logger parameters
use Logger qw(CONSOLE_ON CONSOLE_OFF CONSOLE_TERSE);
my $logger = Logger->new(); # the logger element

# Create the utility object
use UtilLib;
my $ut = UtilLib->new();
my $log_time = $ut->ut_file_name_date_time_stamp();

my $progname = basename $0;

my $result = main();

if ($result) {
    exit(0); # DOS Success Code
} else {
    exit(1); # DOS Error Code
}

#####
# Subroutine to display the usage
sub Usage {
    print "

    Analyze Baseline Records in ALM
    Usage:
        $progname -user <username>
         -pw <password>
         -dbname <database name>
         -dbset <database set>
         -pvob <pvob>
         -baseline <baseline> 
		 -logfile <logfile>
		 -test
		 -help
";
return 0;
}

#### Set up vobs
##Passing in the vob to CQ requires no slashes on the vob, passing in to CC requires a slash on the vob
sub slash_noslash {
	my $pvob = shift;
	my $slashvob;
	my $noslashvob;
	if ($pvob !~ /[\/\\]/) {
	  $slashvob = '\\' .$pvob;
	  $noslashvob = $pvob;
	}
	else{
	  $slashvob = $pvob;
	  $noslashvob = $pvob ;
	  $noslashvob=~ s/\\//g; 
	}
	return ($slashvob, $noslashvob);
}

sub main {
####
# Define the input parameters
my @opts = qw(
    user=s		
    pw=s		
    dbname=s		
    dbset=s		
    pvob=s		
    baseline=s	
    logfile=s
    test
    help
);

my %opts = (
    user => $ENV{ALMUSER},
    pw => $ENV{ALMPW},
    dbname => $ENV{ALMDBNAME},
    dbset => $ENV{ALMDBSET},
    pvob => undef,
    baseline => undef,
    logfile => undef,
);


####
# Get the input parameters
unless (GetOptions(\%opts, @opts)) {
     	$logger->log_print_message("Printing Usage to the command line:");	
      	my $retval = &Usage;
      	return 0;
}

$opts{pw} = "" if ($opts{pw} eq q{""});

#Define the LogFile
if ($opts{logfile}) {
	my $retval;
	my $logdir = dirname($opts{logfile});
	if ( ! -e $logdir) {
		$retval = $ut->ut_create_directory_tree($logdir);
		if ( ! $retval) {
            		$logger->log_print_error( __LINE__ . "Could not create the log file directory: $logdir");
            		$logger->log_print_error( __LINE__ . "Script execution will continue without it.");
        	}
	}
	$retval = $logger->log_set_file_on($opts{logfile});
	if ( ! $retval) {
        	$logger->log_print_error( __LINE__ . "Could not open the log file: $opts{logfile}.");
        	$logger->log_print_error( __LINE__ . "Script execution will continue without it.");
   	 }
	 $logger->log_set_console(CONSOLE_OFF);
	$logger->log_print_message("Logging started at $log_time"); 
}	 



####
# Display the help usage
Usage if ($opts{help}); 

# Display test inputs to log file 
if ($opts{test}){
	$logger->log_print_message("main: Defined user inputs:");
	$logger->log_print_message("main: Defined user login is $opts{user}.");
	$logger->log_print_message("main: Defined dbset is $opts{dbset}.");
	$logger->log_print_message("main: Defined database name is $opts{dbname}.");
	$logger->log_print_message("main: Defined baseline name is $opts{baseline}.");
	$logger->log_print_message("main: Defined Project VOB is $opts{pvob}.");
	if ($opts{logfile}){ $logger->log_print_message("Defined logfile is $opts{logfile}.");}
	return 1;
}

####
# End program if any of the parameters are not defined
if (!defined $opts{user}
  or !defined $opts{pw}
  or !defined $opts{dbname}
  or !defined $opts{dbset}
  or !defined $opts{pvob}
  or !defined $opts{baseline}) {
  $logger->log_print_message("Printing Usage to the command line:");
  my $retval = &Usage();
  return 0;
}
;


####
#### Start a CQ session
my $session = CQPerlExt::CQSession_Build();
# Set session variable 
#$session->SetNameValue("EDIT_BUILD_ACTION", "ACCESS_OK");
#### Logon to the specified database

eval {$session->UserLogon("$opts{user}", "$opts{pw}", "$opts{dbname}", "$opts{dbset}")};

if ($@) {
	$logger->log_print_error(__LINE__ . "main: Login to $opts{dbname} failed: $@"); 
	return 0;
}

$logger->log_print_success("main: Login to $opts{dbname} succeeded.");
#

($slashvob, $noslashvob ) = &slash_noslash($opts{pvob});


# Get baseline details
my $blcq = new get_record_details(session=>$session, type=>'ALMBaseline',fields=>{Name=>$opts{baseline}, PVOB_OrLocation=>$noslashvob},top=>1);
print "Baseline: $opts{baseline}\n";
print "	Activities on Baseline\n";
for my $act ($blcq->get('Activities')) {
	# get the activities details
	print "	Activity $act\n";
	my $acto = new get_record_details(session=>$session, type=>'ALMActivity',fields=>{ID=>$act});

	print "		Tasks the Activity is from\n";
	my @tasks = $acto->get('Task');
	for my $task (@tasks) {
		my $tasko = new get_record_details(session=>$session, type=>'ALMTask',fields=>{ID=>$task});
		my @task_activities = sort $tasko->get('Activities');

		my @ReadyForIntegration ;
		my $ExistingReadyForIntegration = 0;
		my @ReadyForQA ;
		my $ExistingReadyForQA = 0;
		print "\t\ttask $task\n";
		#now that we have a task look at each activity on the task
		print "\t\t\tActivities on this Task with status\n";

		for my $task_act (@task_activities){
				my $task_acto = new get_record_details(session=>$session, type=>'ALMActivity',fields=>{ID=>$task_act});
				my ($state) = $task_acto->get('State');
				my ($type) = $task_acto->get('Type');
				print "\t\t\t$task_act State=$state Type=$type\n";

				# Criteria for activities that need to be created
				# TODO make these commandline options
				$ExistingReadyForIntegration++ if ( $type eq "Ready for Integration" );
				$ExistingReadyForQA++ if ( $type eq "Ready for QA" );
				push(@ReadyForIntegration,$task_act) if ( $type eq "Dev"  and $state eq "Completed" );
				push(@ReadyForQA,$task_act) if ( $type eq "Dev"  and $state eq "Completed" );
				push(@ReadyForQA,$task_act) if ( $type eq "Ready for Integration"  and $state eq "Completed" );
		}

		if (!$ExistingReadyForIntegration) {
			if ( scalar(@task_activities) ==  scalar(@ReadyForIntegration) ) {
				print qq{Creating "Ready for Integration" Activity\n};
				#TODO Error Checking
				my $newact = create_activity($session,"Ready for Integration",$task,$opts{baseline},\@ReadyForIntegration);
				print qq{New Integration Activity: }, $newact->GetDisplayName(), "\n";
				$newact->EditEntity("Activate");
				$newact->Validate();
				$newact->Commit();
			}
			else {
				my $diff = scalar(@task_activities) - scalar(@ReadyForIntegration);
				print qq{"Ready for integration" waiting on $diff Activities\n};
			}
		}
		elsif (!$ExistingReadyForQA){
			if ( @task_activities == @ReadyForQA ) {
				print qq{Creating "Ready for QA" Activity\n};
				#TODO Error Checking
				create_activity($session,"Ready for QA",$task,$opts{baseline},\@ReadyForQA);
			}
			else {
				my $diff = scalar(@task_activities) - scalar(@ReadyForQA);
				print qq{"Ready for QA waiting" on $diff Activities\n};
			}
		}
		else {
			print qq{No new "Ready for Integration" or "Ready for QA" Activity needed\n};
		}

	}
}

} #end main
sub create_activity {
	my $session = shift;
	my $type = shift;
	my $task = shift;
	my $baseline = shift;
	my $task_activities = shift;
	my $activity = new create_record(
		session=>$session,
		type=>'ALMActivity',
		fields => [
			"Headline" => "$type : task $task : baseline $baseline",
			"Task"=>$task,
			"Type"=>$type,
			"Owner"=>"TestLead_ToDo",
			"ActivitiesRelated" => [@$task_activities],
		],

	);
	#TODO Error checking.
	$activity->Validate;
	$activity->Commit;
	return $activity;
}

1;
__END__

